/**
 * @file
 * Provides behaviors for Field UI AJAX.
 */

(function ($, window, Drupal, drupalSettings) {

  'use strict';

  /**
   * Attaches the Ajax behavior to each Ajax form element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.fieldUiAJAX = {
    attach: function (context, settings) {
      var $context = $(context);
      // For ajax links that have "use-ajax-once" we want to remove the ajax
      // behavior after the first click.
      $context.find('.use-ajax-once').once('use-ajax-once').each(function () {
        $(this).on('click', function () {
          var element = $(this);
          element.removeClass('use-ajax use-ajax-once').off('click');
          var hide = element.data('field-ui-hide');
          var show = element.data('field-ui-show');
          if (hide && show) {
            // If it has both a 'hide' and a 'show' we need to make the link a
            // toggle. We add the class here and the behavior will be attached
            // after this each lopp is finished.
            element.addClass('js-field-ui-toggle');
          }
          if (show) {
            processAditionalLinks(show);
          }
        });
      });

      // Behavior for toggle links. We don't use context here because the links
      // can be outside the context.
      $('.js-field-ui-toggle').once('js-field-ui-toggle').each(function () {
        var hide = $(this).data('field-ui-hide');
        var show = $(this).data('field-ui-show');
        $(this).on('click.fieldUiToggle', function (event) {
          $(hide).addClass('js-field-ui-hidden');
          $(show).removeClass('js-field-ui-hidden');
          event.preventDefault();
        });
      });

      // Behavior for tabs.
      var nav = $('nav .secondary');
      var span = $('span.visually-hidden', nav);
      $context.find('.js-field-ui-tabs').once('js-field-ui-tabs').each(function () {
        $(this).on('click.fieldUiTabs', function (event) {
          var element = $(this);
          $('.is-active', nav).removeClass('is-active');
          span.appendTo(element);
          element.addClass('is-active').parent().addClass('is-active');
          event.preventDefault();
        });
      });

      function processAditionalLinks(type) {
        // If we have multiple links that should trigger this, process them
        // all.
        $('.use-ajax-once' + type + '-trigger').each(function () {
          var other_element = $(this);
          other_element.removeClass('use-ajax use-ajax-once').off();
          var hide = other_element.data('field-ui-hide');
          var show = other_element.data('field-ui-show');
          if (hide && show) {
            other_element.addClass('js-field-ui-toggle');
          }
        });
      }
    },

    detach: function (context, settings) {
    }
  };

})(jQuery, this, Drupal, drupalSettings);
