<?php

/**
 * @file
 * Contains \Drupal\field_ui_ajax\Controller\FieldUiAjaxController.
 */

namespace Drupal\field_ui_ajax\Controller;

use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\Controller\EntityController;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\AfterCommand;
use Drupal\Core\Ajax\AppendCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\RestripeCommand;
use Drupal\Core\Ajax\OpenDialogCommand;
use Drupal\field\Entity\FieldConfig;
use Drupal\Core\Entity\HtmlEntityFormController;
use Drupal\field_ui_ajax\Component\Utility\HtmlExtra;

class FieldUiAjaxController extends HtmlEntityFormController {

  public function storageConfigEditForm(RouteMatchInterface $route_match, $field_config, $entity_type_id) {
    $container = \Drupal::getContainer();
    $entityManager = $container->get('entity.manager');
    $form_object = $entityManager->getFormObject('field_storage_config', 'edit');
    $entity = $form_object->getEntityFromRouteMatch($route_match, 'field_storage_config');
    $form_object->setEntity($entity);

    $form_state = new FormState();
    $form_state->addBuildInfo('args', [$field_config]);
    $form = \Drupal::formBuilder()->buildForm($form_object, $form_state);
    if (!HtmlExtra::getIsAjax()) {
      return $form;
    }

    $selector = 'js-' . str_replace(['.', '_'], '-', $field_config);
    $wrapper = '<tr class="' . $selector . '-storage-form"><td colspan="4">';
    $build = [
      '#prefix' => $wrapper,
      '#suffix' => '</td></tr>',
      'messages' => [
        '#type' => 'status_messages',
      ],
      'form' => $form,
    ];

    $response = new AjaxResponse();
    $response->addCommand(new AfterCommand(
      '.' . $selector,
      $build
    ));
    // Hide the table row that triggered the ajax call.
    $response->addCommand(new InvokeCommand(
      '.' . $selector,
      'addClass',
      ['js-field-ui-hidden']
    ));
    $response->addCommand(new RestripeCommand('.js-field-ui-ajax-overview'));
    return $response;
  }

  public function fieldConfigDeleteForm(FieldConfig $field_config) {
    $container = \Drupal::getContainer();
    $entityManager = $container->get('entity.manager');
    $form_object = $entityManager->getFormObject('field_config', 'delete');
    $form_object->setEntity($field_config);

    $form_state = new FormState();
    $form = \Drupal::formBuilder()->buildForm($form_object, $form_state);

    if (!HtmlExtra::getIsAjax()) {
      return $form;
    }

    $selector = 'js-' . str_replace(['.', '_'], '-', $field_config->id());
    $wrapper = '<tr class="' . $selector . '-delete-form"><td colspan="4">';
    $build = [
      '#prefix' => $wrapper,
      '#suffix' => '</td></tr>',
      'messages' => [
        '#type' => 'status_messages',
      ],
      'form' => $form,
    ];

    $response = new AjaxResponse();
    $response->addCommand(new AfterCommand(
      '.' . $selector,
      $build
    ));
    // Hide the table row that triggered the ajax call.
    $response->addCommand(new InvokeCommand(
      '.' . $selector,
      'addClass',
      ['js-field-ui-hidden']
    ));
    $response->addCommand(new RestripeCommand('.js-field-ui-ajax-overview'));
    return $response;
  }

  public function fieldConfigEditForm(FieldConfig $field_config) {
    $container = \Drupal::getContainer();
    $entityManager = $container->get('entity.manager');
    $form_object = $entityManager->getFormObject('field_config', 'edit');
    $form_object->setEntity($field_config);

    $form_state = new FormState();
    $form = \Drupal::formBuilder()->buildForm($form_object, $form_state);

    if (!HtmlExtra::getIsAjax()) {
      return $form;
    }

    $selector = 'js-' . str_replace(['.', '_'], '-', $field_config->id());
    $wrapper = '<div class="' . $selector . '-edit-form">';
    $build = [
      '#prefix' => $wrapper,
      '#suffix' => '</div>',
      'messages' => [
        '#type' => 'status_messages',
      ],
      'form' => $form,
    ];

    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand(
      '.action-links, .tableresponsive-toggle-columns, .js-field-ui-ajax-overview',
      'addClass',
      ['js-field-ui-hidden']
    ));
    $response->addCommand(new AfterCommand(
      '.js-field-ui-ajax-overview',
      $build
    ));
    return $response;
  }

  public function entityViewDisplayPage(RouteMatchInterface $route_match, $entity_type_id, $view_mode_name, $bundle) {
    $container = \Drupal::getContainer();
    $entityManager = $container->get('entity.manager');
    $form_object = $entityManager->getFormObject('entity_view_display', 'edit');
    $entity = $form_object->getEntityFromRouteMatch($route_match, 'entity_view_display');
    $form_object->setEntity($entity);

    $form_state = new FormState();
    $form = \Drupal::formBuilder()->buildForm($form_object, $form_state);

    $selector = $entity_type_id . '-' . $view_mode_name;
    $selector = 'js-view-' . str_replace(['.', '_'], '-', $selector);
    $build = [
      '#prefix' => '<div class="' . $selector . ' js-manage-display-content">',
      '#suffix' => '</div>',
      'dialog' => '<div id="field-ui-ajax-dialog"></div>',
      'messages' => [],
      'form' => $form,
    ];

    if (!HtmlExtra::getIsAjax()) {
      return $build;
    }

    $build['messages'] = ['#type' => 'status_messages'];
    unset($build['dialog']);


    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand(
      '.js-manage-display-content',
      'addClass',
      ['js-field-ui-hidden']
    ));
    $response->addCommand(new AppendCommand(
      '.block-system-main-block',
      $build
    ));
    return $response;
  }

  public function entityFormDisplayPage(RouteMatchInterface $route_match, $entity_type_id, $form_mode_name, $bundle) {

    $container = \Drupal::getContainer();
    $entityManager = $container->get('entity.manager');
    $form_object = $entityManager->getFormObject('entity_form_display', 'edit');
    $entity = $form_object->getEntityFromRouteMatch($route_match, 'entity_form_display');
    $form_object->setEntity($entity);

    $form_state = new FormState();
    $form = \Drupal::formBuilder()->buildForm($form_object, $form_state);

    $selector = $entity_type_id . '-' . $view_mode_name;
    $selector = 'js-form-' . str_replace(['.', '_'], '-', $selector);
    $build = [
      '#prefix' => '<div class="' . $selector . ' js-manage-display-content">',
      '#suffix' => '</div>',
      'dialog' => '<div id="field-ui-ajax-dialog"></div>',
      'messages' => [],
      'form' => $form,
    ];

    if (!HtmlExtra::getIsAjax()) {
      return $build;
    }

    $build['messages'] = ['#type' => 'status_messages'];
    unset($build['dialog']);


    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand(
      '.js-manage-display-content',
      'addClass',
      ['js-field-ui-hidden']
    ));
    $response->addCommand(new AppendCommand(
      '.block-system-main-block',
      $build
    ));
    return $response;
  }

}
