<?php

/**
 * @file
 * Contains \Drupal\field_ui_ajax\Form\FieldConfigAjaxDeleteForm.
 */

namespace Drupal\field_ui_ajax\Form;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\field_ui\Form\FieldConfigDeleteForm;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\field_ui\FieldUI;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\OpenDialogCommand;
use Drupal\Core\Ajax\RestripeCommand;
use Drupal\field_ui_ajax\Component\Utility\HtmlExtra;

/**
 * Provides a form for removing a field from a bundle.
 */
class FieldConfigAjaxDeleteForm extends FieldConfigDeleteForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $form['intro'] = [
      '#weight' => -1000,
      '#markup' => '<h2>' . $this->getQuestion() . '</h2>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   * Add the cancel action and the AJAX submit handler.
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);

    if (HtmlExtra::getIsAjax()) {
      $selector = 'js-' . str_replace(['.', '_'], '-', $this->entity->id());
      $actions['cancel']['#attributes']['class'][] = 'js-field-ui-toggle';
      $actions['cancel']['#attributes']['data-field-ui-show'] = '.' . $selector;
      $actions['cancel']['#attributes']['data-field-ui-hide'] = '.' . $selector . '-delete-form';
      $actions['submit']['#ajax'] = [
        'callback' => '::configAjaxDeleteFormSubmit',
      ];
    }

    return $actions;
  }

  /**
   * Ajax callback for deleting a field.
   *
   * This remove the form and the row in the table on succesful submission.
   */
  public static function configAjaxDeleteFormSubmit(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $selector = 'js-' . $form_state->getValue('field_ui_selector');

    if ($form_state->hasAnyErrors()) {
      $build = [
        '#prefix' => '<div>',
        '#suffix' => '</div>',
        'messages' => [
          '#type' => 'status_messages',
        ],
        'form' => $form,
      ];
      $response->addCommand(new HtmlCommand(
        '.' . $selector . '-delete-form td',
        $build
      ));
    }
    else {
      // Remove previous validation error messages and rows with the form and
      // the field.
      $response->addCommand(new InvokeCommand(
        '.messages, .' . $selector . ', .' . $selector . '-delete-form, .' . $selector . '-edit-form, .' . $selector . '-storage-form',
        'remove'
      ));
      // Show messages in a dialog.
      $response->addCommand(new OpenDialogCommand(
        '#field-ui-ajax-dialog',
        NULL,
        ['#type' => 'status_messages'],
        ['width' => '90%']
      ));
      $response->addCommand(new RestripeCommand('.js-field-ui-ajax-overview'));
    }
    return $response;
  }

}
